@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -501,7 +501,9 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
   { fid_t fid;
 
     if ( (fid = PL_open_foreign_frame()) )
-    { predicate_t pred = PL_predicate("absolute_file_name", 3, "system");
+    { if ( !GD->procedures.absolute_file_name3 )
+	GD->procedures.absolute_file_name3 =
+	  PL_predicate("absolute_file_name", 3, "system");
       term_t av = PL_new_term_refs(3);
       term_t options = PL_copy_term_ref(av+2);
       int rc = TRUE;
@@ -520,7 +522,8 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
 	rc = add_option(options, FUNCTOR_access1, ATOM_execute);
 
       if ( rc ) rc = PL_unify_nil(options);
-      if ( rc ) rc = PL_call_predicate(NULL, cflags, pred, av);
+      if ( rc ) rc = PL_call_predicate(NULL, cflags,
+				       GD->procedures.absolute_file_name3, av);
       if ( rc ) rc = PL_get_nchars(av+1, &len, namep,
 				   CVT_ATOMIC|BUF_STACK|REP_FN);
       if ( rc && strlen(*namep) != len )
@@ -714,7 +717,7 @@ PRED_IMPL("access_file", 2, access_file, 0)
     md = ACCESS_WRITE;
   else if ( m == ATOM_read )
     md = ACCESS_READ;
-  else if ( m == ATOM_execute )
+  else if ( m == ATOM_execute || m == ATOM_search )
     md = ACCESS_EXECUTE;
   else if ( m == ATOM_exist )
     md = ACCESS_EXIST;
@@ -908,7 +911,7 @@ PRED_IMPL("$tmp_file_stream", 4, tmp_file_stream, 0)
   if ( !PL_get_chars(A1, &ext, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) ||
        !PL_get_atom_ex(A2, &encoding) )
     return FALSE;
-  if ( (enc = atom_to_encoding(encoding)) == ENC_UNKNOWN )
+  if ( (enc = PL_atom_to_encoding(encoding)) == ENC_UNKNOWN )
   { if ( encoding == ATOM_binary )
     { enc = ENC_OCTET;
       mode = "wb";
