@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2020, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -63,9 +64,11 @@ is supposed to give the POSIX standard one.
 #endif
 #endif
 
-#include "pl-incl.h"
+#include "pl-os.h"
 #include "pl-ctype.h"
 #include "pl-utf8.h"
+#include "../pl-fli.h"
+#include "../pl-setup.h"
 #include <math.h>
 #include <stdio.h>		/* rename() and remove() prototypes */
 
@@ -126,6 +129,7 @@ static void	initExpand(void);
 static void	cleanupExpand(void);
 static void	initEnviron(void);
 static char    *utf8_path_lwr(char *s, size_t len);
+static void	clean_tmp_dir(void);
 
 #ifndef DEFAULT_PATH
 #define DEFAULT_PATH "/bin:/usr/bin"
@@ -172,6 +176,7 @@ initOs(void)
 void
 cleanupOs(void)
 { cleanupExpand();
+  clean_tmp_dir();
 }
 
 
@@ -637,6 +642,21 @@ free_tmp_symbol(void *name, void *value)
  * encoding.
  */
 
+static atom_t      tmp_aname = NULL_ATOM;
+static const char *tmp_name = NULL;
+
+static void
+clean_tmp_dir(void)
+{ if ( tmp_name )
+  { PL_free((void*)tmp_name);
+    tmp_name = NULL;
+  }
+  if ( tmp_aname )
+  { PL_unregister_atom(tmp_aname);
+    tmp_aname = 0;
+  }
+}
+
 static const char *
 tmp_dir(void)
 { GET_LD
@@ -645,8 +665,6 @@ tmp_dir(void)
   if ( LD )
 #endif
   { atom_t a;
-    static atom_t      tmp_aname = NULL_ATOM;
-    static const char *tmp_name = NULL;
 
     if ( PL_current_prolog_flag(ATOM_tmp_dir, PL_ATOM, &a) )
     { if ( a == tmp_aname )
@@ -658,8 +676,7 @@ tmp_dir(void)
 	if ( (t=PL_new_term_ref()) &&
 	     PL_put_atom(t, a) &&
 	     PL_get_chars(t, &s, CVT_ATOM|REP_FN|BUF_MALLOC) )
-	{ if ( tmp_name ) PL_free((void*)tmp_name);
-	  if ( tmp_aname ) PL_unregister_atom(tmp_aname);
+	{ clean_tmp_dir();
 
 	  tmp_aname = a;
 	  tmp_name = s;
@@ -1095,18 +1112,17 @@ deleteCanonicalDir(CanonicalDir d)
     for(cd=GD->os.dir_table.entries[k]; cd; cd=cd->next)
     { if ( cd->next == d )
       { cd->next = d->next;
-
-	remove_string(d->name);
-	if ( d->canonical != d->name )
-	  remove_string(d->canonical);
-	PL_free(d);
-
-	return;
+	break;
       }
     }
 
-    assert(0);
+    assert(cd);
   }
+
+  remove_string(d->name);
+  if ( d->canonical != d->name )
+    remove_string(d->canonical);
+  PL_free(d);
 }
 
 
@@ -2168,9 +2184,8 @@ Sread_terminal(void *handle, char *buf, size_t size)
 void
 ResetTty(void)
 { GET_LD
-  startCritical;
-  ResetStdin();
 
+  ResetStdin();
   if ( !GD->os.iofunctions.read )
   { GD->os.iofunctions       = *Sinput->functions;
     GD->os.iofunctions.read  = Sread_terminal;
@@ -2180,7 +2195,6 @@ ResetTty(void)
     Serror->functions  = &GD->os.iofunctions;
   }
   LD->prompt.next = TRUE;
-  endCritical;
 }
 
 #ifdef O_HAVE_TERMIO			/* sys/termios.h or sys/termio.h */
@@ -2704,7 +2718,7 @@ argument to wait()
 
 #endif /*HAVE_SYS_WAIT_H*/
 
-const char *
+static const char *
 prog_shell(void)
 { GET_LD
 
@@ -2734,8 +2748,10 @@ System(char *cmd)
   int pid;
   const char *shell = prog_shell();
   int rval;
+#if O_SIGNALS
   void (*old_int)();
   void (*old_stop)();
+#endif
 
   if ( (pid = fork()) == -1 )
   { return PL_error("shell", 2, OsError(), ERR_SYSCALL, "fork");
@@ -2769,10 +2785,12 @@ System(char *cmd)
   { wait_t status;			/* the parent */
     int n;
 
+#if O_SIGNALS
     old_int  = signal(SIGINT,  SIG_IGN);
 #ifdef SIGTSTP
     old_stop = signal(SIGTSTP, SIG_DFL);
 #endif /* SIGTSTP */
+#endif
 
     for(;;)
     {
@@ -2813,10 +2831,12 @@ System(char *cmd)
     }
   }
 
+#if O_SIGNALS
   signal(SIGINT,  old_int);		/* restore signal handlers */
 #ifdef SIGTSTP
   signal(SIGTSTP, old_stop);
 #endif /* SIGTSTP */
+#endif
 
   return rval;
 }
