@@ -3426,14 +3426,6 @@ Swinsock(IOSTREAM *s)
 		 *******************************/
 
 #ifdef HAVE_POPEN
-#ifdef __WINDOWS__
-#include "windows/popen.c"
-
-#undef popen
-#undef pclose
-#define popen(cmd, how) pt_popen(cmd, how)
-#define pclose(fd)	pt_pclose(fd)
-#endif
 
 static ssize_t
 Sread_pipe(void *handle, char *buf, size_t size)
@@ -3774,6 +3766,7 @@ Sopenmem(char **bufp, size_t *sizep, const char *mode)
 { memfile *mf = malloc(sizeof(memfile));
   int flags = SIO_FBUF|SIO_RECORDPOS|SIO_NOMUTEX|SIO_TEXT;
   size_t size;
+  IOSTREAM *s;
 
   if ( !mf )
   { errno = ENOMEM;
@@ -3823,7 +3816,10 @@ Sopenmem(char **bufp, size_t *sizep, const char *mode)
   mf->sizep	= sizep;
   mf->here      = 0;
 
-  return Snew(mf, flags, &Smemfunctions);
+  if ( (s=Snew(mf, flags, &Smemfunctions)) )
+    s->newline = SIO_NL_POSIX;
+
+  return s;
 }
 
 		 /*******************************
@@ -4021,10 +4017,6 @@ SinitStreams(void)
       s->newline = SIO_NL_DOS;
 #endif
     }
-
-#ifdef __WINDOWS__
-    pt_init();				/* init popen() issues */
-#endif
   }
 }
 
