@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2019-2020, University of Amsterdam
+    Copyright (c)  2019-2022, University of Amsterdam
                               VU University Amsterdam
 		              CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -50,6 +50,8 @@
 Event interface
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static void	free_event_callback(event_callback *cb);
+
 #ifdef O_PLMT
 #define INIT_LIST_LOCK(l) recursiveMutexInit(&(l)->lock)
 #define DELETE_LIST_LOCK(l) recursiveMutexDelete(&(l)->lock)
@@ -78,6 +80,7 @@ const event_type PL_events[] =
   GEVENT(PLEV_FRAMEFINISHED,    ATOM_frame_finished,   1, onframefinish),
   GEVENT(PLEV_UNTABLE,		ATOM_untable,          1, onuntable),
 #ifdef O_PLMT
+  GEVENT(PLEV_THREAD_START,     ATOM_thread_start,     1, onthreadstart),
   GEVENT(PLEV_THREAD_EXIT,      ATOM_thread_exit,      1, onthreadexit),
   LEVENT(PLEV_THIS_THREAD_EXIT, ATOM_this_thread_exit, 0, onthreadexit),
 #endif
@@ -304,6 +307,7 @@ PRED_IMPL("prolog_unlisten", 2, prolog_unlisten, 0)
 		if ( !list->head )
 		  list->tail = NULL;
 	      }
+	      free_event_callback(ev);
 	      continue;
 	    }
 	  } else if ( ev->closure.term )
@@ -370,6 +374,17 @@ destroy_event_list(event_list **listp)
   }
 }
 
+void
+cleanupEvents(void)
+{ const event_type *ep;
+
+  for(ep=PL_events; ep->name; ep++)
+  { if ( !ep->local )
+      destroy_event_list(ep->location);
+  }
+}
+
+
 
 #define call_event_list(list, argc, argv) LDFUNC(call_event_list, list, argc, argv)
 static int
@@ -625,6 +640,7 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
       break;
     }
 #ifdef O_PLMT
+    case PLEV_THREAD_START:
     case PLEV_THREAD_EXIT:
     { PL_thread_info_t *info = va_arg(args, PL_thread_info_t*);
 
