@@ -46,6 +46,7 @@
 #include "pl-fli.h"
 #include "pl-modul.h"
 #include "pl-supervisor.h"
+#include "pl-wam.h"
 
 static void	fix_ssu(p_reload *r, Clause clause);
 
@@ -237,18 +238,21 @@ clearSourceAdmin(atom_t sf_name)
 { GET_LD
   int rc = FALSE;
   fid_t fid;
-  static predicate_t pred = NULL;
+  predicate_t pred;
 
-  if ( !pred )
-    pred = PL_predicate("$clear_source_admin", 1, "system");
+  if ( GD->cleaning == CLN_DATA )
+    return TRUE;
+
+  pred = _PL_predicate("$clear_source_admin", 1, "system",
+		       &GD->procedures.clear_source_admin1);
 
   if ( (fid=PL_open_foreign_frame()) )
   { term_t name = PL_new_term_ref();
 
     PL_put_atom(name, sf_name);
-    startCritical;			/* block signals */
+    startCritical();			/* block signals */
     rc = PL_call_predicate(MODULE_system, PL_Q_NODEBUG, pred, name);
-    endCritical;
+    rc = endCritical() && rc;
 
     PL_discard_foreign_frame(fid);
   }
@@ -260,12 +264,13 @@ clearSourceAdmin(atom_t sf_name)
 static atom_t
 destroySourceFile(SourceFile sf)
 { if ( sf->magic == SF_MAGIC )
-  { SourceFile f;
-    atom_t name;
+  { atom_t name;
+    SourceFile f;
 
     sf->magic = SF_MAGIC_DESTROYING;
     f = deleteHTable(GD->files.table, (void*)sf->name);
     assert(f);
+    (void)f;
     name = sf->name;
     putSourceFileArray(sf->index, NULL);
     if ( GD->files.no_hole_before > sf->index )
@@ -775,7 +780,7 @@ unloadFile(SourceFile sf)
 
 static int
 unloadFile(SourceFile sf)
-{ ListCell cell, next;
+{ ListCell cell;
   size_t deleted = 0;
   int rc;
 
@@ -802,14 +807,9 @@ unloadFile(SourceFile sf)
     }
   }
   DEBUG(MSG_UNLOAD, Sdprintf("Removed %ld clauses\n", (long)deleted));
+  (void)deleted;
 
-				      /* cleanup the procedure list */
-  for(cell = sf->procedures; cell; cell = next)
-  { next = cell->next;
-    freeHeap(cell, sizeof(struct list_cell));
-  }
-  sf->procedures = NULL;
-
+  freeList(&sf->procedures);
   delAllModulesSourceFile__unlocked(sf);
   UNLOCKSRCFILE(sf);
 
@@ -872,7 +872,6 @@ PRED_IMPL("$unload_file", 1, unload_file, 0)
 		 *	    RECONSULT		*
 		 *******************************/
 
-static void	fix_discontiguous(p_reload *r);
 static void	fix_metapredicate(p_reload *r);
 static void	fix_det(p_reload *r);
 
@@ -922,6 +921,7 @@ startReconsultFile(SourceFile sf)
 	  cl->generation.erased = r->reload_gen;
 	}
 	release_def(def);
+	clear(def, P_DISCONTIGUOUS);		/* will be reinstantiated */
       }
       if ( true(def, P_AUTOLOAD) )
       { clear(def, P_AUTOLOAD);			/* should be be more selective? */
@@ -946,8 +946,9 @@ find_clause(ClauseRef cref, gen_t generation)
 }
 
 
+#define advance_clause(r) LDFUNC(advance_clause, r)
 static void
-advance_clause(p_reload *r ARG_LD)
+advance_clause(DECL_LD p_reload *r)
 { ClauseRef cref;
 
   if ( (cref = r->current_clause) )
@@ -979,15 +980,16 @@ copy_clause_source(Clause dest, Clause src)
 }
 
 
+#define keep_clause(r, clause) LDFUNC(keep_clause, r, clause)
 static ClauseRef
-keep_clause(p_reload *r, Clause clause ARG_LD)
+keep_clause(DECL_LD p_reload *r, Clause clause)
 { ClauseRef cref = r->current_clause;
   Clause keep = cref->value.clause;
 
   keep->generation.erased = GEN_MAX;
   copy_clause_source(keep, clause);
   freeClause(clause);
-  advance_clause(r PASS_LD);
+  advance_clause(r);
 
   return cref;
 }
@@ -1006,7 +1008,7 @@ equal_clause(Clause cl1, Clause cl2)
 
 
 int
-reloadHasClauses(SourceFile sf, Procedure proc ARG_LD)
+reloadHasClauses(DECL_LD SourceFile sf, Procedure proc)
 { p_reload *reload;
 
   if ( sf->reload && (reload=lookupHTable(sf->reload->procedures, proc)) )
@@ -1038,8 +1040,9 @@ isRedefinedProcedure(Procedure proc, gen_t gen)
 }
 
 
+#define reloadContext(sf, proc) LDFUNC(reloadContext, sf, proc)
 static p_reload *
-reloadContext(SourceFile sf, Procedure proc ARG_LD)
+reloadContext(DECL_LD SourceFile sf, Procedure proc)
 { p_reload *reload;
 
   if ( !(reload = lookupHTable(sf->reload->procedures, proc)) )
@@ -1054,7 +1057,7 @@ reloadContext(SourceFile sf, Procedure proc ARG_LD)
     if ( true(def, P_THREAD_LOCAL|P_FOREIGN) )
     { set(reload, P_NO_CLAUSES);
     } else if ( isRedefinedProcedure(proc, global_generation()) )
-    { definition_ref *dref = pushPredicateAccessObj(def PASS_LD);
+    { definition_ref *dref = pushPredicateAccessObj(def);
 
       if ( !dref )
       { freeHeap(reload, sizeof(*reload));
@@ -1082,7 +1085,7 @@ reloadContext(SourceFile sf, Procedure proc ARG_LD)
 
 
 ClauseRef
-assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
+assertProcedureSource(DECL_LD SourceFile sf, Procedure proc, Clause clause)
 { if ( sf && sf->reload )
   { p_reload *reload;
     Definition def = proc->definition;
@@ -1092,19 +1095,18 @@ assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
 
     sf->reload->number_of_clauses++;
 
-    if ( !(reload = reloadContext(sf, proc PASS_LD)) )
+    if ( !(reload = reloadContext(sf, proc)) )
     { freeClause(clause);
       return NULL;
     }
 
     if ( reload->number_of_clauses++ == 0 )
-    { fix_discontiguous(reload);
-      fix_det(reload);
+    { fix_det(reload);
       fix_ssu(reload, clause);
     }
 
     if ( true(reload, P_NEW|P_NO_CLAUSES) )
-      return assertProcedure(proc, clause, CL_END PASS_LD);
+      return assertProcedure(proc, clause, CL_END);
 
     if ( (cref = reload->current_clause) )
     { ClauseRef cref2;
@@ -1113,7 +1115,7 @@ assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
       { DEBUG(MSG_RECONSULT_CLAUSE,
 	      Sdprintf("  Keeping clause %d\n",
 		       clauseNo(cref->value.clause, reload->generation)));
-	return keep_clause(reload, clause PASS_LD);
+	return keep_clause(reload, clause);
       }
 
       set(reload, P_MODIFIED);
@@ -1149,7 +1151,7 @@ assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
 	  DEBUG(MSG_RECONSULT_CLAUSE,
 		Sdprintf("  Keeping clause %d\n",
 			 clauseNo(cref2->value.clause, reload->generation)));
-	  return keep_clause(reload, clause PASS_LD);
+	  return keep_clause(reload, clause);
 	}
       }
       release_def(def);
@@ -1157,12 +1159,12 @@ assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
       DEBUG(MSG_RECONSULT_CLAUSE,
 	    Sdprintf("  Inserted before clause %d\n",
 		     clauseNo(cref->value.clause, reload->generation)));
-      if ( (cref2 = assertProcedure(proc, clause, cref PASS_LD)) )
+      if ( (cref2 = assertProcedure(proc, clause, cref)) )
 	cref2->value.clause->generation.created = sf->reload->reload_gen;
 
       return cref2;
     } else
-    { if ( (cref = assertProcedure(proc, clause, CL_END PASS_LD)) )
+    { if ( (cref = assertProcedure(proc, clause, CL_END)) )
 	cref->value.clause->generation.created = sf->reload->reload_gen;
       DEBUG(MSG_RECONSULT_CLAUSE, Sdprintf("  Added at the end\n"));
 
@@ -1174,7 +1176,7 @@ assertProcedureSource(SourceFile sf, Procedure proc, Clause clause ARG_LD)
   { sf->number_of_clauses++;
   }
 
-  return assertProcedure(proc, clause, CL_END PASS_LD);
+  return assertProcedure(proc, clause, CL_END);
 }
 
 
@@ -1205,15 +1207,15 @@ associateSource(SourceFile sf, Procedure proc)
 #define P_ATEND	(P_VOLATILE|P_PUBLIC|P_ISO|P_NOPROFILE|P_NON_TERMINAL)
 
 int
-setAttrProcedureSource(SourceFile sf, Procedure proc,
-		       unsigned attr, int val ARG_LD)
+setAttrProcedureSource(DECL_LD SourceFile sf, Procedure proc,
+		       unsigned attr, int val)
 { if ( val && (attr&PROC_DEFINED) )
     associateSource(sf, proc);
 
   if ( sf->reload )
   { p_reload *reload;
 
-    if ( !(reload = reloadContext(sf, proc PASS_LD)) )
+    if ( !(reload = reloadContext(sf, proc)) )
       return FALSE;
 
     if ( val )
@@ -1259,8 +1261,9 @@ check_ssu(p_reload *r)
 }
 
 
+#define fix_attributes(sf, def, r) LDFUNC(fix_attributes, sf, def, r)
 static void
-fix_attributes(SourceFile sf, Definition def, p_reload *r ARG_LD)
+fix_attributes(DECL_LD SourceFile sf, Definition def, p_reload *r)
 { if ( false(def, P_MULTIFILE) )
     def->flags = (def->flags & ~P_ATEND) | (r->flags & P_ATEND);
   else
@@ -1270,15 +1273,6 @@ fix_attributes(SourceFile sf, Definition def, p_reload *r ARG_LD)
 }
 
 
-static void
-fix_discontiguous(p_reload *r)
-{ Definition def = r->predicate;
-
-  if ( true(def, P_DISCONTIGUOUS) && false(r, P_DISCONTIGUOUS) )
-    clear(def, P_DISCONTIGUOUS);
-}
-
-
 static void
 fix_det(p_reload *r)
 { Definition def = r->predicate;
@@ -1311,15 +1305,15 @@ fix_ssu(p_reload *r, Clause clause)
 
 
 int
-setMetapredicateSource(SourceFile sf, Procedure proc,
-		       arg_info *args ARG_LD)
+setMetapredicateSource(DECL_LD SourceFile sf, Procedure proc,
+		       arg_info *args)
 { associateSource(sf, proc);
 
   if ( sf->reload )
   { p_reload *reload;
     size_t i, arity = proc->definition->functor->arity;
 
-    if ( !(reload = reloadContext(sf, proc PASS_LD)) )
+    if ( !(reload = reloadContext(sf, proc)) )
       return FALSE;
 
     if ( !reload->args )
@@ -1496,6 +1490,8 @@ delete_old_predicates(SourceFile sf)
 	prev->next = cell->next;
       else
 	sf->procedures = cell->next;
+
+      freeHeap(cell, sizeof(*cell));
     } else
     { prev = cell;
     }
@@ -1503,8 +1499,9 @@ delete_old_predicates(SourceFile sf)
 }
 
 
+#define delete_pending_clauses(sf, def, r) LDFUNC(delete_pending_clauses, sf, def, r)
 static void
-delete_pending_clauses(SourceFile sf, Definition def, p_reload *r ARG_LD)
+delete_pending_clauses(DECL_LD SourceFile sf, Definition def, p_reload *r)
 { ClauseRef cref;
   sf_reload *rl = sf->reload;
 
@@ -1532,8 +1529,9 @@ delete_pending_clauses(SourceFile sf, Definition def, p_reload *r ARG_LD)
 }
 
 
+#define end_reconsult_proc(sf, proc, r) LDFUNC(end_reconsult_proc, sf, proc, r)
 static size_t
-end_reconsult_proc(SourceFile sf, Procedure proc, p_reload *r ARG_LD)
+end_reconsult_proc(DECL_LD SourceFile sf, Procedure proc, p_reload *r)
 { size_t dropped_access = 0;
 
   DEBUG(MSG_RECONSULT_CLAUSE,
@@ -1542,16 +1540,16 @@ end_reconsult_proc(SourceFile sf, Procedure proc, p_reload *r ARG_LD)
   if ( false(r, P_NEW|P_NO_CLAUSES) )
   { Definition def = proc->definition;
 
-    delete_pending_clauses(sf, def, r PASS_LD);
-    fix_attributes(sf, def, r PASS_LD);
-    reconsultFinalizePredicate(sf->reload, def, r PASS_LD);
+    delete_pending_clauses(sf, def, r);
+    fix_attributes(sf, def, r);
+    reconsultFinalizePredicate(sf->reload, def, r);
     if ( true(r, P_CHECK_SSU) )
       check_ssu(r);
   } else
   { dropped_access++;
     if ( true(r, P_NO_CLAUSES) )
     { Definition def = proc->definition;
-      fix_attributes(sf, def, r PASS_LD);
+      fix_attributes(sf, def, r);
     }
   }
   if ( r->args )
@@ -1577,7 +1575,7 @@ endReconsult(SourceFile sf)
 	      { Procedure proc = n;
 		p_reload *r = v;
 
-		accessed_preds -= end_reconsult_proc(sf, proc, r PASS_LD);
+		accessed_preds -= end_reconsult_proc(sf, proc, r);
 	      });
 
     popNPredicateAccess(accessed_preds);
@@ -1639,13 +1637,13 @@ flush_procedure(SourceFile sf, Procedure proc)
     { if ( false(r, P_NEW|P_NO_CLAUSES) )
       { Definition def = proc->definition;
 
-	delete_pending_clauses(sf, def, r PASS_LD);
-	fix_attributes(sf, def, r PASS_LD);
-	reconsultFinalizePredicate(reload, def, r PASS_LD);
+	delete_pending_clauses(sf, def, r);
+	fix_attributes(sf, def, r);
+	reconsultFinalizePredicate(reload, def, r);
       }
     } else
     { delete_old_predicate(sf, proc);
-      (void)reloadContext(sf, proc PASS_LD);
+      (void)reloadContext(sf, proc);
     }
   }
 
@@ -1811,7 +1809,7 @@ PRED_IMPL("$clause_from_source", 4, clause_from_source, 0)
   LOCKSRCFILE(of);
   for(cell = of->procedures; cell; cell = cell->next)
   { Procedure proc = cell->value;
-    Definition def = proc->definition;
+    Definition def = getProcDefinition(proc);
 
     if ( def && false(def, P_FOREIGN) )
     { ClauseRef cref;
@@ -1868,8 +1866,9 @@ out:
  * subsequent changes to the predicate are _immediate_.
  */
 
+#define flush_predicate(pred) LDFUNC(flush_predicate, pred)
 static int
-flush_predicate(term_t pred ARG_LD)
+flush_predicate(DECL_LD term_t pred)
 { SourceFile sf;
   Procedure proc;
   Module m = LD->modules.source;
@@ -1895,7 +1894,7 @@ static
 PRED_IMPL("$flush_predicate", 1, flush_predicate, 0)
 { PRED_LD
 
-  return flush_predicate(A1 PASS_LD);
+  return flush_predicate(A1);
 }
 
 
