@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2021, University of Amsterdam
+    Copyright (c)  1985-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -218,7 +218,7 @@ first. The last byte has its 0x80 mask set.
 #define CLAUSE_UNIT_CLAUSE 0x01
 #define CLAUSE_SSU_COMMIT  0x02
 #define CLAUSE_SSU_CHOICE  0x04
-
+#define CLAUSE_HEAD_TERMS  0x08
 
 static char saveMagic[] = "SWI-Prolog state (www.swi-prolog.org)\n";
 static char qlfMagic[]  = "SWI-Prolog .qlf file\n";
@@ -1299,6 +1299,7 @@ loadClauseFlags(wic_state *state, Clause cl, int skip)
     if ( (flags&CLAUSE_UNIT_CLAUSE) ) lflags |= UNIT_CLAUSE;
     if ( (flags&CLAUSE_SSU_COMMIT) )  lflags |= SSU_COMMIT_CLAUSE;
     if ( (flags&CLAUSE_SSU_CHOICE) )  lflags |= SSU_CHOICE_CLAUSE;
+    if ( (flags&CLAUSE_HEAD_TERMS) )  lflags |= CL_HEAD_TERMS;
 
     cl->flags = lflags;
   }
@@ -1679,10 +1680,10 @@ runInitialization(SourceFile sf)
   { GET_LD
     fid_t fid = PL_open_foreign_frame();
     term_t av = PL_new_term_refs(2);
-    static predicate_t pred = NULL;
+    predicate_t pred;
 
-    if ( !pred )
-      pred = PL_predicate("$run_initialization", 2, "system");
+    pred = _PL_predicate("$run_initialization", 2, "system",
+			 &GD->procedures.drun_initialization2);
 
     PL_put_atom(av+0, sf->name);
     PL_put_nil( av+1);
@@ -2593,6 +2594,7 @@ clauseFlags(const Clause clause)
   if ( true(clause, UNIT_CLAUSE) )       flags |= CLAUSE_UNIT_CLAUSE;
   if ( true(clause, SSU_COMMIT_CLAUSE) ) flags |= CLAUSE_SSU_COMMIT;
   if ( true(clause, SSU_CHOICE_CLAUSE) ) flags |= CLAUSE_SSU_CHOICE;
+  if ( true(clause, CL_HEAD_TERMS) )     flags |= CLAUSE_HEAD_TERMS;
 
   return flags;
 }
@@ -3378,12 +3380,12 @@ popPathTranslation(wic_state *state)
       if ( (tr=old->translated) )
       { GET_LD
         path_translated *n;
-	static predicate_t pred = NULL;
+	predicate_t pred;
 	fid_t fid = PL_open_foreign_frame();
 	term_t av = PL_new_term_refs(2);
 
-	if ( !pred )
-	  pred = PL_predicate("$translated_source", 2, "system");
+	pred = _PL_predicate("$translated_source", 2, "system",
+			     &GD->procedures.dtranslated_source2);
 
 	for(; tr; tr=n)
 	{ n = tr->next;
